/*
SDX: Documentary System in XML.
Copyright (C) 2000, 2001, 2002  Ministere de la culture et de la communication (France), AJLSM

Ministere de la culture et de la communication,
Mission de la recherche et de la technologie
3 rue de Valois, 75042 Paris Cedex 01 (France)
mrt@culture.fr, michel.bottin@culture.fr

AJLSM, 17, rue Vital Carles, 33000 Bordeaux (France)
sevigny@ajlsm.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
or connect to:
http://www.fsf.org/copyleft/gpl.html
 */
package fr.gouv.culture.sdx.search.lucene.query;

import fr.gouv.culture.sdx.documentbase.LuceneDocumentBase;
import fr.gouv.culture.sdx.exception.SDXException;
import fr.gouv.culture.sdx.exception.SDXExceptionCode;
import fr.gouv.culture.sdx.framework.Framework;
import fr.gouv.culture.sdx.search.lucene.DateField;
import fr.gouv.culture.sdx.search.lucene.Field;
import fr.gouv.culture.sdx.search.lucene.filter.Criteria;
import fr.gouv.culture.sdx.search.lucene.filter.Filter;
import fr.gouv.culture.sdx.utils.Bits;
import fr.gouv.culture.sdx.utils.Date;
import fr.gouv.culture.sdx.utils.SdxObjectImpl;
import fr.gouv.culture.sdx.utils.Utilities;
import fr.gouv.culture.sdx.utils.constants.Node;
import org.apache.avalon.framework.logger.Logger;
import org.apache.cocoon.ProcessingException;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.search.Hits;
import org.apache.regexp.RE;
import org.apache.regexp.RESyntaxException;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import java.io.IOException;
import java.text.Collator;
import java.util.*;

/*
 *	Une liste de termes de la base de documents.
 *
 *	Les termes sont des valeurs uniques de champs. On peut construire une
 *	liste de tous les termes, une liste de termes qui respectent une
 *	expression r�guli�re (dans le but de g�rer la troncature), une
 *	liste de termes pour un champ, ainsi qu'une liste de termes pour
 *	un champ en fonction d'une requ�te de recherche.
 */

/** A list of terms for a document base.
 *
 * The terms are single values of fields. One can build one
 * list of all the terms, a list of terms which respect one
 * regular expression (with an aim of managing truncation), one
 * list of terms for a field, as well as a list of terms for
 * a field according to a search acceptRequest.
 *
 */
public class Terms extends SdxObjectImpl implements fr.gouv.culture.sdx.search.Terms {

    /** Avalon logger to write information. */
    private static org.apache.avalon.framework.logger.Logger logger;

    /** The ordered list of terms. */
    private TreeMap termList;

    /** The number of document to show per page. */
    private int hitsPerPage = HITS_PER_PAGE;

    /** The maximum number of pages. */
    private int nbPages;

    /** The default number of documents to show per page. */
    public static final int HITS_PER_PAGE = 20;

    /** The fields of filters. */
    private String[] fieldFilters;

    /** The values of filters. */
    private String[] valueFilters;

    /** The identifier of the term list. */
    private String id;

    /** THe filter used for the document base. */
    private Filter filter;

    /** The object of comparison for sorting  */
    private static Collator sortCollator;

    /** The Lucene indices where to find the terms. */
    private SearchLocations searchLocations;

    /**
     *	Cr�e une liste de l'ensemble des termes de la base de documents.
     *
     *	@param	db	L'information sur la base de documents.
     */
/*
	public Terms(LuceneIndex index) throws IOException
	{

		this.index = index;

		IndexReader reader = index.getReader();
		TermEnum terms = reader.terms();
		reader.close();

		// Il faut les trier
		initCollator(index);
		termList = new TreeMap(sortCollator);
		while ( terms.next() )
			termList.put(terms.term().text(), new TermInfo(reader, terms.term()));
		nbPages = countPages();
	}
 */

    /**
     *	Builds a term list.
     *
     *  @param	sLocs	The SearchLocations object (indices to be searched).
     *	@param	field	The field name.
     */
    public void setUp(SearchLocations sLocs, String field) throws SDXException {
        if (sLocs == null) throw new SDXException(logger, SDXExceptionCode.ERROR_SEARCHLOCATIONS_NULL, null, null);

        if (!Utilities.checkString(field)) throw new SDXException(logger, SDXExceptionCode.ERROR_FIELD_NAME_NULL, null, null);
        setFieldFilter(field);
        //interning the field name for better performance with lucene
        String internedfieldName = field.intern();

        this.searchLocations = sLocs;

        initCollator(searchLocations.getField(field));
        termList = new TreeMap(sortCollator);
        IndexReader reader = null;
        TermEnum ts = null;
        try {
            int nbReaders = searchLocations.size();
            for (int i = 0; i < nbReaders; i++) {
                reader = searchLocations.getIndexReader(i);
                if (reader != null) {
                    ts = reader.terms(new Term(internedfieldName, ""));
//                    Term t;
//                    String text;
                    boolean collecting = false;
                    if (ts != null) {
                        do {
                            if (ts.term() != null && ts.term().field() == internedfieldName) {
                                collecting = true;
                                TermInfo ti = (TermInfo) termList.get(ts.term().text());
                                if (ti != null)
                                    ti.update(reader, ts.term());
                                else {
                                    ti = new TermInfo();
                                    ti.enableLogging(logger);
                                    ti.setUp(reader, ts.term());
                                    termList.put(ts.term().text(), ti);
                                }
                                /*
                                                            TermInfo ti = new TermInfo();
                                                            ti.enableLogging(logger);
                                                            ti.setUp(reader, ts.term());
                                                            termList.put(ts.term().text(), ti);
                                */

                            } else if (collecting) break;
                        } while (ts.next());
                    }
                }
            }
        } catch (IOException e) {
            String[] args = new String[1];
            args[0] = e.getMessage();
            throw new SDXException(null, SDXExceptionCode.ERROR_SETUP_TERMS, args, e);
        } finally {
            //freeing resources
            if (ts != null) {
                try {
                    ts.close();
                } catch (IOException e) {
                    String[] args = new String[1];
                    args[0] = e.getMessage();
                    throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_TERMENUM_CLOSE, args, e);
                }
            }
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    String[] args = new String[1];
                    args[0] = e.getMessage();
                    throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_READER_CLOSE, args, e);
                }
            }
        }

        nbPages = countPages();
    }

    /**
     *	Builds a term list.
     *
     *  @param	sLocs	The SearchLocations object (indices to be searched).
     *	@param	field	The field name.
     *  @param  str     The term
     */
    public void setUp(SearchLocations sLocs, String field, String str) throws SDXException {
        //TODO?:if str == null we call the more simple constructor, but should we really do this or throw and exception here?-rbp
        if (sLocs == null) throw new SDXException(logger, SDXExceptionCode.ERROR_SEARCHLOCATIONS_NULL, null, null);

        if (!Utilities.checkString(field)) throw new SDXException(logger, SDXExceptionCode.ERROR_FIELD_NAME_NULL, null, null);
        setFieldFilter(field);
        //interning the field name for better performance with lucene
        String internedfieldName = field.intern();

        if (str == null)
            setUp(sLocs, field);
        else {
            this.searchLocations = sLocs;

            initCollator(searchLocations.getField(field));
            termList = new TreeMap(sortCollator);
            IndexReader reader = null;
            TermEnum ts = null;
            RE re = null;
            try {
                re = getRE(str);
            } catch (RESyntaxException e) {
                String[] args = new String[1];
                args[0] = e.getMessage();
                throw new SDXException(null, SDXExceptionCode.ERROR_SETUP_TERMS, args, e);
            }
            int nbReaders = searchLocations.size();
            for (int i = 0; i < nbReaders; i++) {
                reader = searchLocations.getIndexReader(i);
                if (reader != null) {
                    try {
                        ts = reader.terms(new Term(internedfieldName, ""));
                        Term t = null;
                        String text;
                        boolean collecting = false;
                        if (ts != null) {
                            do {
                                if (ts.term() != null)
                                    t = ts.term();
                                if (t != null && t.field() == internedfieldName) {
                                    collecting = true;
                                    text = t.text();
                                    if (re.match(text)) {
                                        TermInfo ti = (TermInfo) termList.get(text);
                                        if (ti != null)
                                            ti.update(reader, ts.term());
                                        else {
                                            ti = new TermInfo();
                                            ti.enableLogging(logger);
                                            ti.setUp(reader, ts.term());
                                            termList.put(text, ti);
                                        }
                                        /*
                                                                        TermInfo ti = new TermInfo();
                                                                        ti.enableLogging(logger);
                                                                        ti.setUp(reader, t);
                                                                        termList.put(text, ti);
                                        */
                                    }
                                } else if (collecting) break;
                            } while (ts.next());
                        }
                    } catch (IOException e) {
                        String[] args = new String[1];
                        args[0] = e.getMessage();
                        throw new SDXException(null, SDXExceptionCode.ERROR_SETUP_TERMS, args, e);
                    } finally {
                        //freeing resources
                        if (ts != null) {
                            try {
                                ts.close();
                            } catch (IOException e) {
                                String[] args = new String[1];
                                args[0] = e.getMessage();
                                throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_TERMENUM_CLOSE, args, e);
                            }
                        }
                        if (reader != null) {
                            try {
                                reader.close();
                            } catch (IOException e) {
                                String[] args = new String[1];
                                args[0] = e.getMessage();
                                throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_READER_CLOSE, args, e);
                            }
                        }
                    }
                }
            }

            nbPages = countPages();
        }
    }

    protected void setFieldFilter(String field) {
        String[] fields = {field};
        fieldFilters = fields;
    }

    /**
     *	Builds a term list.
     *
     *  @param	results	The Results object (indices to be searched).
     *	@param	field	The field name.
     */
    public void setUp(Results results, String field, String value) throws SDXException {
        setFieldFilter(field);
        String[] fields = new String[1];
        fields[0] = field;
        String[] values = new String[1];
        values[0] = value;
        setUp(results, fields, values);
    }


    public void setUp(Results results, String[] fields, String[] values) throws SDXException {
        if (results != null) {
            this.searchLocations = results.getSearchLocations();
            if (searchLocations == null) throw new SDXException(logger, SDXExceptionCode.ERROR_SEARCHLOCATIONS_NULL, null, null);
            if (fields == null) throw new SDXException(logger, SDXExceptionCode.ERROR_FIELD_NAME_NULL, null, null);

            // If there is more than one Reader in the searchlocation, then log a warning
            // TODO Warn: specific message...
            if (this.searchLocations.size() > 1) Utilities.logWarn(logger, "Only the first search location will be used...", null);
            // We store the fields and values
            fieldFilters = fields;
            termList = getTerms(results.getHits(), fields, values);
            nbPages = countPages();
        }
    }


    /**
     * Builds a lits of terms from multiple criterias.
     *
     * <p>IMPORTANT : only one search location is actually used.
     * TODO : fix this...
     *
     * <p>Criterias are field=value pairs. This method lets a
     * developer creates hierarchical lists.
     *
     * <p>There should be one value less than fields; the last
     * field will then be the field to return values from.
     *
     * <p>For instance, if the first field is 'region', the
     * second is 'department' and the third is 'city', and
     * the first value is 'Aquitaine' and the second value is
     * 'Gironde', you will get a list of cities that appear in
     * documents that also have region=Aquitaine and
     * department=Gironde.
     *
     * @param  sLocs       Where to find the terms (may not bu null).
     * @param  fields      The list of fields (may not be null).
     * @param  values      The list of values (there should be one less value than fields).
     */
    public void setUp(SearchLocations sLocs, String[] fields, String[] values) throws SDXException {
        if (sLocs == null) throw new SDXException(logger, SDXExceptionCode.ERROR_SEARCHLOCATIONS_NULL, null, null);
        if (fields == null) throw new SDXException(logger, SDXExceptionCode.ERROR_FIELD_NAME_NULL, null, null);

        this.searchLocations = sLocs;

        // If there is more than one Reader in the searchlocation, then log a warning
        // TODO : specific message...
        if (sLocs.size() > 1) Utilities.logWarn(logger, "Only the first search location will be used...", null);

        // We store the fields and values
        fieldFilters = fields;
        valueFilters = values;

        // Consider a null values as an array of length 0, or no filter...
        if (values == null) values = new String[0];

        // We only build the terms if there is one more field than values
        if (fields.length == (values.length + 1)) {

            // We'll build a simple filter, by ANDing all field=value
            // pairs except the last field.

            BitSet documentSet = null;
            IndexReader r = sLocs.getIndexReader(0);
            if (fields.length > 1) {
                filter = new Filter(Filter.BOOLEAN_OPERATOR_AND);

                for (int i = 0; i < values.length; i++) {
                    Criteria c = new Criteria();
                    c.enableLogging(logger);
                    c.setUp(sLocs.getField(fields[i]), values[i]);
                    filter.add(c);
                }
                documentSet = filter.bits(r);
            }


            // Then we build the list of values of the last field
            // with at least one document in the bitset.
            try {
                termList = getTerms(r, sLocs.getField(fields[fields.length - 1]), documentSet);
            } finally {
                if (r != null) {
                    try {
                        r.close();
                    } catch (IOException e) {
                        String[] args = new String[1];
                        args[0] = e.getMessage();
                        throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_READER_CLOSE, args, e);
                    }
                }
            }
            nbPages = countPages();

        } else {
            // Here, we don't mett the criteria that there must be one more field than value
            // TODO : log a warning...
        }
    }

    /**
     *	Cr�e une liste filtr�e de termes.
     *
     *	Le filtre sp�cifie les documents dont les termes
     *	doivent �tre extraits.
     *
     *	@param	db  	L'informations sur la base de documents.
     *	@param	field	Le champ.
     *	@param	filter	Le filtre.
     */
/*
    public Terms(SearchLocations sLocs, String field, Filter filter) throws SDXException, IOException {
        this.searchLocations = sLocs;

        //TODO?		if ( field == null || db == null || filter == null ) throw new SDXException("fr", "Impossible de constuire une liste de termes.");
        {
            this.filter = filter;
            IndexReader r = index.getReader();
            BitSet documentSet = filter.bits(r);

            // Ensuite, on doit trouver les valeurs du champ qui ont
            // au moins un document dans le filtre, et trouver le nombre
            // de documents.

            termList = getTerms(r, field, documentSet);
            r.close();
            nbPages = countPages();
        }
    }
*/

    /*
     * Cr�e une liste de termes en fonction d'une cha�ne avec troncature.
     *
     */
    /**Creates a list of terms according to a chain with truncation
     *
     *  <p>
     * A logger must be set and then the Terms must be setUp.
     *
     * @see #enableLogging
     * @see #setUp
     */
    public Terms() {
    }


    /**
     *	G�n�re une sortie XML de la liste de termes.
     *
     *	@param	factory		Le document h�te pour cette sortie XML.
     */
/*
	public Element toDOM(Document factory) throws IOException
	{
		return toDOM(factory, -1);
	}
 */

    /**
     *	Returns an XML representation of this term list.
     *
     *	@param	hdl	    The ContentHandler to feed with SAX events.
     */
    public void toSAX(ContentHandler hdl) throws SAXException, ProcessingException {
        toSAX(hdl, -1);
    }

    /**
     *	G�n�re une sortie XML de la liste de termes.
     *
     *	@param	factory		Le document h�te pour cette sortie XML.
     *	@param	page		Le num�ro de la page � sortir.
     */
/*
	public Element toDOM(Document factory, int page) throws IOException
	{
		String nsURI = Utilities.getSDXNamespaceURI();
		String nsPrefix = Utilities.getSDXNamespacePrefix();

		if ( page < 1 ) page = 1;
		if ( page > nbPages ) page = nbPages;
		int sIdx = ( (page - 1) * hitsPerPage );
		if ( sIdx < 0 ) sIdx = 0;

		int eIdx = ( page * hitsPerPage ) - 1;
		if ( eIdx >= termList.size() ) eIdx = termList.size() - 1;

		Element top = factory.createElementNS(nsURI, nsPrefix + ":terms");
		top.setAttribute("xmlns:" + nsPrefix, nsURI);
                //forward compatibility : use of old attribute name (take care !)
		if ( fieldFilters != null && fieldFilters.length > 0 ) top.setAttribute("code", fieldFilters[fieldFilters.length - 1]);
		if ( id != null ) top.setAttribute("id", id);

		// Les informations sur le nombre de pages
		top.setAttribute("start", String.valueOf(sIdx+1));
		top.setAttribute("end", String.valueOf(eIdx+1));
		top.setAttribute("nb", String.valueOf(termList.size()));
		top.setAttribute("nbPages", String.valueOf(nbPages));
		top.setAttribute("currentPage", String.valueOf(page));

		// On va ajouter des informations sur les filtres utilis�s.
		if ( filter != null ) top.appendChild(filter.toDOM(factory));

		if ( hitsPerPage != 0 )
		{
			Element termEl;
			if ( termList != null )
			{
				Iterator it = termList.keySet().iterator();
				int i=0;
				while ( it.hasNext() )
				{
					i++;

					if ( i > sIdx && i <= eIdx+1 )
					{
						fr.gouv.culture.sdx.query.lucene.TermInfo term = (fr.gouv.culture.sdx.query.lucene.TermInfo)termList.get((String)it.next());
						termEl = factory.createElementNS(nsURI, nsPrefix + ":term");
						termEl.setAttribute("docFreq", String.valueOf(term.getDocFrequency()));
						String fieldname = term.getField();
						String content;
						if ( db != null && db.getFieldType(fieldname) == DBField.DATE ) content = Utilities.formatDate(DateField.stringToDate(term.getContent()));
						else
							content = term.getContent();
						termEl.setAttribute("escapedValue", java.net.URLEncoder.encodeURL(content));
						termEl.setAttribute("field", fieldname);
						termEl.setAttribute("value", content);
						termEl.setAttribute("no", String.valueOf(i));

						// S'il y a un seul document, alors on retourne aussi son identificateur.
						if ( term.getDocFrequency() == 1 ) termEl.setAttribute("docId", term.getDocId());

						top.appendChild(termEl);
					}
					else if ( i > eIdx+1 ) break; else it.next();
				}
			}
		}
		return top;
	}
 */

    /**
     *	Returns an XML representation of this term list.
     *
     *	@param	hdl	    The ContentHandler to feed with SAX events.
     *	@param	page	The number of the page to show.
     */
    public void toSAX(ContentHandler hdl, int page) throws SAXException, ProcessingException {
        String sdxNsUri = Framework.SDXNamespaceURI;
        String sdxNsPrefix = Framework.SDXNamespacePrefix;

        if (page < 1) page = 1;
        if (page > nbPages) page = nbPages;
        int sIdx = ((page - 1) * hitsPerPage);
        if (sIdx < 0) sIdx = 0;

        int eIdx = (page * hitsPerPage) - 1;
        if (termList != null && eIdx >= termList.size()) eIdx = termList.size() - 1;

        //Creation of local variables which are later passed into startElement() and endElement() methods-rbp13/03/02
        String localName = Node.Name.TERMS;
        String qName = sdxNsPrefix + ":" + localName;
        AttributesImpl atts = new AttributesImpl();


        if (fieldFilters != null && fieldFilters.length > 0) {
            atts.addAttribute("", Node.Name.NAME, Node.Name.NAME, Node.Type.CDATA, fieldFilters[fieldFilters.length - 1]);
            atts.addAttribute("", Node.Name.FIELD, Node.Name.FIELD, Node.Type.CDATA, fieldFilters[fieldFilters.length - 1]);
        }

        // Les informations sur le nombre de pages
        //TODO?:should we also add the field name here or other information?-rbp
        if (id != null) atts.addAttribute("", Node.Name.QID, Node.Name.QID, Node.Type.CDATA, id);
        atts.addAttribute("", Node.Name.PAGE, Node.Name.PAGE, Node.Type.CDATA, String.valueOf(page));
        atts.addAttribute("", Node.Name.HPP, Node.Name.HPP, Node.Type.CDATA, String.valueOf(hitsPerPage));
        atts.addAttribute("", Node.Name.PAGES, Node.Name.PAGES, Node.Type.CDATA, String.valueOf(nbPages));
        if (termList != null)
            atts.addAttribute("", Node.Name.NB, Node.Name.NB, Node.Type.CDATA, String.valueOf(termList.size()));
        atts.addAttribute("", Node.Name.START, Node.Name.START, Node.Type.CDATA, String.valueOf(sIdx + 1));
        atts.addAttribute("", Node.Name.END, Node.Name.END, Node.Type.CDATA, String.valueOf(eIdx + 1));

        // for backward compatibility but not documented
        if (id != null) atts.addAttribute("", Node.Name.ID, Node.Name.ID, Node.Type.CDATA, id);
        atts.addAttribute("", Node.Name.CURRENT_PAGE, Node.Name.CURRENT_PAGE, Node.Type.CDATA, String.valueOf(page));
        atts.addAttribute("", Node.Name.NB_PAGES, Node.Name.NB_PAGES, Node.Type.CDATA, String.valueOf(nbPages));

        //startElement() method is called for "terms" and local variables are passed-rbp12/03/02
        hdl.startElement(sdxNsUri, localName, qName, atts);

        // On va ajouter des informations sur les filtres utilis�s.
        if (filter != null) filter.toSAX(hdl);

        if (hitsPerPage != 0) {
            if (termList != null) {
                Iterator it = termList.keySet().iterator();
                int i = 0;
                while (it.hasNext()) {
                    i++;

                    if (i > sIdx && i <= eIdx + 1) {
                        TermInfo term = (TermInfo) termList.get(it.next());
                        //Creation of local variables which are later passed into startElement() and endElement() methods-rbp12/03/02
                        String subLocalName = Node.Name.TERM;
                        String subQName = sdxNsPrefix + ":" + subLocalName;
                        AttributesImpl subAtts = new AttributesImpl();
                        subAtts.addAttribute("", Node.Name.NO, Node.Name.NO, Node.Type.CDATA, String.valueOf(i));

                        String fieldname = term.getField();
                        String content;

                        int fieldType = -1;
                        try {
                            fieldType = searchLocations.getFieldType(fieldname);
                        } catch (SDXException sdxE) {
                            throw new SAXException(sdxE.getMessage(), sdxE);
                        }
                        if (searchLocations != null && fieldType == Field.DATE)
                            content = Date.formatDate(DateField.stringToDate(term.getContent()));
                        else
                            content = term.getContent();

                        String esc = null;
                        esc = Utilities.encodeURL(content, super.encoding);
                        // try { esc=java.net.URLEncoder.encodeURL(content, Framework.URL_ENCODING); } catch (java.io.UnsupportedEncodingException e) {} // 1.4
                        subAtts.addAttribute("", Node.Name.VALUE, Node.Name.VALUE, Node.Type.CDATA, content);
                        subAtts.addAttribute("", Node.Name.ESCAPED_VALUE, Node.Name.ESCAPED_VALUE, Node.Type.CDATA, esc);
                        subAtts.addAttribute("", Node.Name.FIELD, Node.Name.FIELD, Node.Type.CDATA, fieldname);


                        subAtts.addAttribute("", Node.Name.DOCS, Node.Name.DOCS, Node.Type.CDATA, String.valueOf(term.getDocFrequency()));
                        // S'il y a un seul document, alors on retourne aussi son identificateur.
                        if (term.getDocFrequency() == 1) {
                            subAtts.addAttribute("", Node.Name.ID, Node.Name.ID, Node.Type.CDATA, term.getDocId());
                            subAtts.addAttribute("", Node.Name.DB_ID, Node.Name.DB_ID, Node.Type.CDATA, term.getDbId());
                            subAtts.addAttribute("", Node.Name.APPID, Node.Name.APPID, Node.Type.CDATA, term.getAppId());
                            subAtts.addAttribute("", Node.Name.DOC_ID, Node.Name.DOC_ID, Node.Type.CDATA, term.getDocId());
                        }
                        // not documented but preserved for backward compatibility
                        subAtts.addAttribute("", Node.Name.DOC_FREQ, Node.Name.DOC_FREQ, Node.Type.CDATA, String.valueOf(term.getDocFrequency()));
                        //startElement() method is called for "term" and local variables are passed-rbp12/03/02
                        hdl.startElement(sdxNsUri, subLocalName, subQName, subAtts);

                        //endElement() method is called for "term" and local variables are passed-rbp12/03/02
                        hdl.endElement(sdxNsUri, subLocalName, subQName);

                    } else if (i > eIdx + 1) break; else it.next();
                }
            }
        }
        //endElement() method is called for "terms" and local variables are passed-rbp12/03/02
        hdl.endElement(sdxNsUri, localName, qName);
    }

    /**
     *	Returns a list of the terms.
     */
    public TreeMap getList() {
        return termList;
    }

    /**
     *	Constructs a regular expression from a term of the acceptRequest.
     *
     *	@param	str		The term.
     */
    private static RE getRE(String str) throws RESyntaxException {
        if (str == null) return null;
        StringBuffer st = new StringBuffer("^");
        for (int i = 0; i < str.length(); i++) {
            char c = str.charAt(i);
            switch (c) {
                case '*':
                    st.append("(.*)");
                    break;

                case '?':
                    st.append("(.?)");
                    break;

                case '(':
                case ')':
                case '.':
                case '\\':
                case '[':
                case ']':
                case '^':
                case '$':
                case '+':
                case '{':
                case '}':
                case '|':
                    st.append('\\');
                    st.append(c);
                    break;

                default:
                    st.append(c);
            }
        }
        st.append("$");
        return new RE(st.toString());
    }

    /*
     *	Retourne une liste de termes en fonction d'un champ et d'une valeur.
     *
     *	La valeur peut contenir des masques * et ?. Tous les termes de ce champ
     *	qui respectent la valeur masqu�e seront retourn�s. Si aucun terme ne
     *	satisfait le crit�re, on retourne une liste vide.
     *
     *	@param	r		Un objet pour lire l'index.
     *	@param	f	Le nom du champ
     *	@param	pattern	La valeur masqu�e.
     */
    /**
     * Returns a list of terms according to a field and a value.
     *
     * The value can contain masks and?. All terms of this field
     * which respects the masked value will be turned over. If no term
     * satisfied the criterion, an empty list is returned.
     *
     * @param r     The index reader
     * @param f     The field
     * @param pattern   The pattern
     * @throws SDXException
     */
    public static TreeMap getTerms(IndexReader r, Field f, String pattern) throws SDXException {
        TreeMap ret = null;

        if (r == null || f == null || pattern == null) return ret;
        String internedfieldName = "";
        if (Utilities.checkString(f.getCode()))
        //interning the field name for better performance with lucene
            internedfieldName = f.getCode().intern();

        if (f.getLocale() != null)
            ret = new TreeMap(Collator.getInstance(f.getLocale()));
        else
        //TODO?:is this acceptable default behavior?-rbp
            ret = new TreeMap(Collator.getInstance(new Locale("fr", "FR")));

        TermEnum ts = null;
        try {
            RE re = getRE(pattern);
            ts = r.terms(new Term(internedfieldName, ""));
            boolean collecting = false;
            if (ts != null) {
                do {
                    if (ts.term() != null && ts.term().field() == internedfieldName) {
                        collecting = true;
                        if (ts.term() != null && re != null && re.match(ts.term().text())) {
                            // First check if the term is already in the list
                            TermInfo ti = (TermInfo) ret.get(ts.term().text());
                            if (ti != null)
                                ti.update(r, ts.term());
                            else {
                                ti = new TermInfo();
                                ti.enableLogging(logger);
                                ti.setUp(r, ts.term());
                                ret.put(ts.term().text(), ti);
                            }
                        }
                    } else if (collecting) break;
                } while (ts.next());
            }
        } catch (RESyntaxException e) {
            String[] args = new String[1];
            args[0] = e.getMessage();
            throw new SDXException(null, SDXExceptionCode.ERROR_GET_TERMS, args, e);
        } catch (IOException e) {
            String[] args = new String[1];
            args[0] = e.getMessage();
            throw new SDXException(null, SDXExceptionCode.ERROR_GET_TERMS, args, e);
        } finally {
            try {
                //freeing resources
                if (ts != null) ts.close();
            } catch (IOException e) {
                String[] args = new String[2];
                args[0] = f.getCode();
                args[1] = e.getMessage();
                throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_TERMENUM_CLOSE, args, e);
            }
        }

        return ret;
    }

    /**
     *	Retourne une liste de valeurs pour un champ.
     *
     *	@param	r		L'objet pour lire l'index.
     *	@param	field	Le nom du champ.
     */
/*  Not used anymore ? MS
    public static TreeMap getTerms(IndexReader r, String field) throws SDXException, IOException {
        TreeMap ret = new TreeMap(Collator.getInstance(new Locale("fr", "FR")));
        TermEnum ts = r.terms();
        boolean collecting = false;
        while (ts.next()) {
            if (ts.term().field().equals(field)) {
                collecting = true;
                TermInfo ti = (TermInfo)ret.get(ts.term().text());
                if ( ti != null ) ti.update(r, ts.term());
                else {
                    ti = new TermInfo();
                    ti.enableLogging(logger);
                    ti.setUp(r, ts.term());
                    ret.put(ts.term().text(), ti);
                }
            } else if (collecting) break;
        }
        return ret;
    }
*/

    /**
     * Builds a list of values for a filtered f.
     *
     *	@param	hits		The results.
     *	@param	fields	        The fields in question.
     *	@param	values	        The values for the fields in question.
     */
    private TreeMap getTerms(Hits hits, String[] fields, String[] values) throws SDXException {

        TreeMap ret = new TreeMap();
        if (hits == null || hits.length() == 0) return ret; // TODO : is the the better solution? Harmonize with other lists of terms...
        Hashtable fieldMaps = new Hashtable();
        for (int i = 0; i < hits.length(); i++) {
            try {
                Document hitDoc = hits.doc(i);
                if (hitDoc != null) {
                    for (int j = 0; j < fields.length; j++) {
                        Field f = this.searchLocations.getField(fields[j]);
                        String lfname = f.getCode();
                        if (f != null) {
                            TreeMap map = null;
                            if (fieldMaps.size() > 0 && fieldMaps.containsKey(lfname))
                                map = (TreeMap) fieldMaps.get(lfname);
                            else {
                                if (f.getCollator() == null)
                                    map = new TreeMap(Collator.getInstance(new Locale("fr", "FR"))); // TODO : is it OK?
                                else
                                    map = new TreeMap(f.getCollator());
                            }
                            String[] fieldsValues = hitDoc.getValues(lfname);
                            if (fieldsValues != null) {
                                HashSet termsCollected = new HashSet();
                                for (int k = 0; k < fieldsValues.length; k++) {
                                    String lftext = fieldsValues[k];
                                    if (Utilities.checkString(lftext)) {
                                        if (values == null || values.length == 0 || (j < values.length && (!Utilities.checkString(values[j]) || lftext.equals(values[j])))) {
                                            if (!termsCollected.contains(lftext)) {
                                                TermInfo ti = (TermInfo) map.get(lftext);
                                                if (ti != null) {
                                                    ti.update(lfname, lftext);
                                                } else {
                                                    ti = new TermInfo();
                                                    ti.enableLogging(logger);
                                                    ti.setUp(lfname, lftext);
                                                    ti.update(lfname, lftext);
                                                    ti.setDocId(hitDoc.getField(LuceneIndex.ID_FIELD).stringValue());
                                                    ti.setDbId(hitDoc.getField(LuceneDocumentBase.INTERNAL_FIELD_NAME_SDXDBID).stringValue());
                                                    ti.setAppId(hitDoc.getField(LuceneDocumentBase.INTERNAL_FIELD_NAME_SDXAPPID).stringValue());
                                                    map.put(lftext, ti);
                                                }
                                                termsCollected.add(lftext);
                                            }
                                        }
                                    }
                                }
                            }
                            if (map != null) fieldMaps.put(lfname, map);
                        }
                    }
                }
            } catch (IOException e) {
                String[] args = new String[1];
                args[0] = e.getMessage();
                new SDXException(null, SDXExceptionCode.ERROR_GET_TERMS, args, e);
            }
        }


        Enumeration maps = fieldMaps.elements();
        if (maps != null) {
            while (maps.hasMoreElements()) {
                TreeMap outmap = (TreeMap) maps.nextElement();
                ret.putAll(outmap);
            }
            maps = null;
        }
        return ret;
    }


    /**
     * Builds a list of values for a filtered f.
     *
     *	@param	r		The reader.
     *	@param	f	The f.
     *	@param	docs	A list of documents as a filter.
     */
    private TreeMap getTerms(IndexReader r, Field f, BitSet docs) throws SDXException {

        if (f == null) return new TreeMap(); // TODO : is the the better solution? Harmonize with other lists of terms...

        //interning the f name for better performance with lucene
        String internedfieldName = "";
        if (Utilities.checkString(f.getCode())) internedfieldName = f.getCode().intern();

        TreeMap ret;
        if (f.getCollator() == null)
            ret = new TreeMap(Collator.getInstance(new Locale("fr", "FR"))); // TODO : is it OK?
        else
            ret = new TreeMap(f.getCollator());

        TermEnum ts = null;
        try {
            ts = r.terms(new Term(internedfieldName, ""));
            boolean collecting = false;
            BitSet termDocumentSet;
            int nbDocs;
            do {
                if (ts.term() != null && ts.term().field() == internedfieldName) {
                    collecting = true;

                    // We build the bitset for this term and compare
                    // it with the one we have, and then count what's
                    // left.

                    termDocumentSet = getDocumentSet(r, ts.term());
                    if (docs != null) termDocumentSet.and(docs);
                    nbDocs = Bits.countBits(termDocumentSet);
                    if (nbDocs > 0) {
                        TermInfo ti = (TermInfo) ret.get(ts.term().text());
                        if (ti != null)
                            ti.update(r, ts.term());
                        else {
                            ti = new TermInfo();
                            ti.enableLogging(logger);
                            ti.setUp(r, ts.term());
                            ret.put(ts.term().text(), ti);
                        }
                    }
                } else if (collecting) break;
            } while (ts.next());
            return ret;
        } catch (IOException e) {
            String[] args = new String[1];
            args[0] = e.getMessage();
            throw new SDXException(null, SDXExceptionCode.ERROR_GET_TERMS, args, e);
        } finally {
            try {
                //freeing resources
                if (ts != null) ts.close();
            } catch (IOException e) {
                String[] args = new String[2];
                args[0] = f.getCode();
                args[1] = e.getMessage();
                throw new SDXException(null, SDXExceptionCode.ERROR_LUCENE_TERMENUM_CLOSE, args, e);
            }
        }
    }


    /**
     * Returns a list of the documents associated with a term within the index.
     *
     *	@param	r		The index reader.
     *	@param	term	The term for searching.
     */
    public static BitSet getDocumentSet(IndexReader r, Term term) throws IOException {
        BitSet bits = new BitSet(r.numDocs());
        TermDocs docs = r.termDocs(term);
        while (docs.next()) bits.set(docs.doc());
        return bits;
    }

    /**
     *	Ecrit la liste de termes en XML dans un writer.
     *
     *	@param	writer		Le writer o� �crire.
     *	@param	page		La page � envoyer.
     */
/*
	public void toXML(Writer writer, int page) throws SDXException
	{
		XMLSerializer ser = new XMLSerializer(writer, new OutputFormat("xml", "UTF-8", true));
		try
		{
			ser.serialize(toDOM(new DocumentImpl(), page));
		}
		catch ( IOException e )
		{
			throw new SDXException(e, "fr", "Impossible de s�rialiser le document XML");
		}
	}
 */

    /**
     *	Ecrit la liste de termes en XML dans un flux de sortie.
     *
     *	@param	os		Le flux o� �crire.
     *	@param	page		La page � envoyer.
     */
/* No need for this anymore as it is being handled in the servlet class, GetTerms.java.-rbp21/03/02
	public void toXML(OutputStream os, int page) throws SDXException
	{
		XMLSerializer ser = new XMLSerializer(os, new OutputFormat("xml", "UTF-8", true));
		try
		{
			ser.serialize(toDOM(new DocumentImpl(), page));
		}
		catch ( IOException e )
		{
			throw new SDXException(e, "fr", "Impossible de s�rialiser le document XML");
		}
	}
 */

    /**
     *	Counts the number of pages for these terms.
     */
    private int countPages() {
        if (termList == null)
            return 0;
        else if (termList.size() == 0)
            return 0;
        else {
            if (hitsPerPage == 0) return 1;
            float temp = ((float) termList.size() / (float) hitsPerPage);
            if (temp != Math.round(temp))
                return (int) (Math.round(Math.floor(temp) + 1));
            else
                return Math.round(temp);
        }
    }

    /**
     *	Sets the identifier for this list of terms.
     *
     *	@param	id		The id.
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     *	Get the number of hits per page.
     */
    public int getHitsPerPage() {
        return hitsPerPage;
    }

    /**
     *	Set the number of hits per page.
     *
     *	@param	nb		The number of hits.
     */
    public void setHitsPerPage(int nb) {
        if (nb < 0)
            setAllHits();
        else {
            hitsPerPage = nb;
            nbPages = countPages();
        }
    }

    /**
     *	Initializes the collator used for sorting .
     *
     *	@param	field  	The field from the document base.
     */
    private void initCollator(Field field) {
        if (field != null) sortCollator = field.getCollator();
    }

    /**
     *	Indicate to return all the results on only one page.
     */
    public void setAllHits() {
        if (termList != null)
            hitsPerPage = termList.size();
        nbPages = 1;
    }

    /**
     * Sets the logger.
     *
     * @param   logger      The logger to use.
     */
    public void enableLogging(Logger logger) {
        this.logger = logger;
    }


    public SearchLocations getSearchLocations() {
        return searchLocations;
    }


}
