/*
SDX: Documentary System in XML.
Copyright (C) 2000, 2001, 2002  Ministere de la culture et de la communication (France), AJLSM

Ministere de la culture et de la communication,
Mission de la recherche et de la technologie
3 rue de Valois, 75042 Paris Cedex 01 (France)
mrt@culture.fr, michel.bottin@culture.fr

AJLSM, 17, rue Vital Carles, 33000 Bordeaux (France)
sevigny@ajlsm.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
or connect to:
http://www.fsf.org/copyleft/gpl.html
*/
package fr.gouv.culture.sdx.utils.rdbms;

import fr.gouv.culture.sdx.utils.Utilities;
import org.apache.avalon.excalibur.datasource.DataSourceComponent;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.ComponentSelector;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;

/**
 * Created by IntelliJ IDEA.
 * User: rpandey
 * Date: Feb 25, 2003
 * Time: 2:30:44 PM
 * To change this template use Options | File Templates.
 */

/**This class will handle connections manaagement within the scope
 * of the coocoon jdbc connection pool
 *
 */
public abstract class DataSourceComponentBacked extends JDBC implements Composable, Configurable {
/*This class should in the configure() method get the dbselector from cocoon
 and  then manage all requests for DataSourceComponentS
*/

    protected ComponentManager manager = null;

    /** The data source identifier. */
    protected String dsi;

    /** The component selector. */
    protected ComponentSelector dbSelector;

    /**String representation of the "repository" attribute name "dsi", for dataSourceIdentifier (name of connection in cocoon.xconf). */
    protected final String ATTRIBUTE_DSI = "dsi";


    /** Compose the object so that we get the <code>Component</code>s we need from the
     * <code>ComponentManager</code>.
     * @param manager   The component manager from Cocoon that allow us to acquire a database selector component.
     * @throws org.apache.avalon.framework.component.ComponentException
     */
    public void compose(ComponentManager manager) throws ComponentException {
        //setting the manager
        this.manager = manager;

        if (this.manager != null)
            this.dbSelector = (ComponentSelector) this.manager.lookup(DataSourceComponent.ROLE + "Selector");
    }

    public void configure(Configuration configuration) throws ConfigurationException {
        // We will need to keep the datasource identifier here
        this.dsi = configuration.getAttribute(ATTRIBUTE_DSI, null);
    }

    /**Get's the datasource component from Cocoon using the dbSelector and the dsi
     * class variables
     *
     * @return
     * @throws ComponentException
     */
    protected DataSourceComponent getDataSourceComponent() throws ComponentException {
        if (this.dbSelector == null || !Utilities.checkString(this.dsi)) return null;
        return (DataSourceComponent) dbSelector.select(dsi);
    }

    /***Releases the data source component using the dbSelector
     *
     * @param dsc   The component to release
     */
    protected void releaseDataSourceComponent(DataSourceComponent dsc) {
        if (this.dbSelector != null && dsc != null) this.dbSelector.release(dsc);
    }


}
